import sys
from oslo_config import cfg
from forward_server.main import main as server_main
from forward_client.forward_client import main as client_main


run_type_opt = [
    cfg.StrOpt('run_type',default='server',help='run type is server or client'),
]
cfg.CONF.register_cli_opts(run_type_opt)

def main():


    cfg.CONF(sys.argv[1:])

    print cfg.CONF.run_type
    if cfg.CONF.run_type == 'server':
        #server_main(args.cfg_file)
        server_main()
    elif cfg.CONF.run_type == 'client':
        #client_main(args.cfg_file)
        client_main()
    else:
        print 'run_type error'


if __name__ == '__main__':

    sys.exit(main())



import sys
import os
import pyinotify
import ConfigParser

# PortMapper save the relationship between server's out port and client's inner network's ip:port
# PortMapper parse the mapper file to get these relationships
# PortMapper monitor the mapper file,if the mapper file changed when server process running,PortMapper can received
# a notify event,PortMapper refresh the new relationship immediately

class PortMapper(pyinotify.ProcessEvent):
    class Coninfo(object):
        def __init__(self,ip,port,tag='default'):
            self.ip = ip
            self.port = port
            self.tag = tag
        def __eq__(self, other):
            return self.ip == other.ip and self.port == other.port and self.tag == other.tag

        def __ne__(self, other):
            return not self.__eq__(other)

    def __init__(self,map_file_path):
        # map out port to inner port,like:{1234:Coninfo('192.168.1.5',1234)}
        #self._port_to_info = {1234:self.Coninfo('192.168.184.128',4321)}
        #self._port_to_info = {1234:self.Coninfo('127.0.0.1',2222)}
        #self._port_to_info = {1234:self.Coninfo('127.0.0.1',80),
        #                       1235:self.Coninfo('127.0.0.1',22),
        #                       1236: self.Coninfo('127.0.0.1',4321)}

        self._map_file_path = map_file_path
        self._port_to_info = {}
        self._refresh_port_info()

    def _refresh_port_info(self):
        cf = ConfigParser.ConfigParser()
        temp_info = {}
        try:
            cf.readfp(open(self._map_file_path, 'rb'))
            mappers = cf.items('MAPPER')
            for item in mappers:
                outer_port = int(item[0].lstrip().rstrip())
                inner_info = item[1].lstrip().rstrip().split(':')
                inner_ip = inner_info[0].lstrip().rstrip()
                inner_port = int(inner_info[1].lstrip().rstrip())
                temp_info[outer_port] = self.Coninfo(inner_ip,inner_port)

            tags = cf.items('TAG')
            for item in tags:
                outer_port = int(item[0].lstrip().rstrip())
                tag = item[1].lstrip().rstrip()
                if temp_info.has_key(outer_port):
                    temp_info[outer_port].tag = tag
        except Exception,e:
            print e
            raise e

        new_ports = [port for port in set(temp_info.keys()) - set(self._port_to_info.keys())]
        deld_ports = [port for port in set(self._port_to_info.keys()) - set(temp_info.keys())]
        changed_ports = []

        for port in set(self._port_to_info.keys()) - set(deld_ports):
            if self._port_to_info[port] != temp_info[port]:
                changed_ports.append(port)

        self._port_to_info = temp_info
        return new_ports,deld_ports,changed_ports

    def flush_to_file(self):
        cf = ConfigParser.ConfigParser()
        cf.add_section('MAPPER')
        cf.add_section('TAG')
        try:
            for port,info in self._port_to_info.items():
                cf.set('MAPPER',str(port),"%s:%d"%(info.ip,info.port))
                cf.set('TAG',str(port),info.tag)

            cf.write(open(self._map_file_path,'w'))
        except Exception,e:
            raise e

    def get_outer_ports(self):
        return [port for port in self._port_to_info]

    def get_port_info(self,port):
        if self._port_to_info.has_key(port):
            return self._port_to_info[port]
        else:
            return None

    def get_inner_info_by_out_port(self,out_port):
        if self._port_to_info.has_key(out_port):
            return self._port_to_info[out_port].ip,self._port_to_info[out_port].port,self._port_to_info[out_port].tag
        else:
            return None,None,None

class FileMonitorPortMapper(PortMapper):
    def __init__(self,map_file_path,call_back):
        super(FileMonitorPortMapper,self).__init__(map_file_path)
        self.__callback = call_back
        self.__wm = pyinotify.WatchManager()
        self.__notifier = pyinotify.Notifier(self.__wm,self)
        self.__wm.add_watch(os.path.dirname(map_file_path), pyinotify.IN_MODIFY)

    def watch_event(self,time_out_ms):
        try:
            self.__notifier.process_events()
            if self.__notifier.check_events(time_out_ms):
                self.__notifier.read_events()
                return True
            else:
                return False
        except Exception,e:
            print 'process_events exception'
            print e.message

    def process_IN_MODIFY(self, event):
        mod_file = os.path.join(event.path, event.name)
        if mod_file == self._map_file_path:
            print  "Modify file: %s " % os.path.join(event.path, event.name)
            new_ports,deld_ports,changed_ports = self._refresh_port_info()

            self.__callback(new_ports, deld_ports, changed_ports)

class InterfaceDriverPortMapper(PortMapper):
    def __init__(self, map_file_path, call_back):
        super(InterfaceDriverPortMapper,self).__init__(map_file_path)
        self.__callback = call_back
        
    def create_new_port(self,port,mapper_ip,mapper_port,mapper_tag):
        if self._port_to_info.has_key(port):
            raise Exception('Create port mapper failed,port:' + str(port) + ' already exists')

        self._port_to_info[port] = self.Coninfo(mapper_ip,mapper_port,mapper_tag)

        self.__callback(new_ports = [port])

        self.flush_to_file()

    def delete_port(self,port):
        if not self._port_to_info.has_key(port):
            raise Exception('Port:' + str(port) + ' not exists')

        self._port_to_info.pop(port)

        self.__callback(deld_ports = [port])

        self.flush_to_file()



import flow_static
import time

class Worker(object):
    def __init__(self,worker_id,address,connector):
        self._worker_id = worker_id
        self._address = address
        self._connector = connector
        self._recv_flow_static = flow_static.FlowStatic()
        self._send_flow_static = flow_static.FlowStatic()

    def get_worker_id(self):
        return self._worker_id

    def get_address(self):
        return self._address

    def get_worker_static_info(self):
        info = {}
        info['id'] = self._worker_id
        local_time = time.localtime(self._recv_flow_static.get_init_time())
        info['init_time'] = "%d-%d-%d %d:%d:%d"%(local_time[0],local_time[1],local_time[2],
                                                 local_time[3],local_time[4],local_time[5])

        info['duration_time'] = int(time.time() - self._recv_flow_static.get_init_time())

        info['ip'] = self._address[0]
        info['port'] = self._address[1]


        info['recv_flow_statistics'] = self._recv_flow_static.get_total_flow()
        info['recv_real_time_speed'] = self._recv_flow_static.get_real_speed()
        info['recv_average_speed'] = self._recv_flow_static.get_average_speed()

        info['send_flow_statistics'] = self._send_flow_static.get_total_flow()
        info['send_real_time_speed'] = self._send_flow_static.get_real_speed()
        info['send_average_speed'] = self._send_flow_static.get_average_speed()

        return info


import api
import info_collection
import json

class ChannelApi(api.RestApiHandler):
    def _get(self, request,response):
        params = self._get_params(request)

        if len(params) == 0:
            #get all port info
            result = self._get_port_info(None)
        elif params.has_key('port'):
            #get specified port info
            result = self._get_port_info(int(params['port']))
        elif params.has_key('channel_id'):
            #get specified channel info
            result = self._get_channel_info(int(params['channel_id']))
        response.body = result

    def _get_port_info(self,port = None):
        result = info_collection.InfoCollection.get_instance().get_channel_info_by_port(port)

        return json.dumps(result)

    def _get_channel_info(self,channel_id):
        collection = info_collection.InfoCollection.get_instance()
        return json.dumps(collection.get_outer_worker_info(channel_id))

    def _delete(self,request,response):
        params = self._get_params(request)

        try:
            if params.has_key('channel_id'):
                info_collection.InfoCollection.get_instance().close_outer_worker(int(params['channel_id']))
            elif params.has_key('channel_ids'):
                channels_list = json.loads(params['channel_ids'])
                for channel_id in channels_list:
                    info_collection.InfoCollection.get_instance().close_outer_worker(int(channel_id))
        except Exception, e:
            return [str({'result': 'failed', 'reason': e.message})]

        response.body = json.dumps({'result': 'success'})


    @classmethod
    def factory(cls, global_conf, **kwargs):
        return cls()


from common import connector
from common import ring_buffer
from common import forward_event
from common import forward_data
import select
import logging
import data_handler
from enum import Enum
import time
import worker
logger = logging.getLogger('my_logger')

class InnerWorker(worker.Worker):
    class State(Enum):
        NONE = 0
        PRE_WORKING = 1
        WORKING = 2
        CLOSED = 3
        DONE = 4
    def __init__(self,worker_id,inner_socket,address,north_interface_channel,connector_change_callback):
        super(InnerWorker,self).__init__(worker_id,address,connector.Connector(inner_socket))
        self.__state = self.State.NONE
        self.__north_interface_channel = north_interface_channel
        self.__data_handler = data_handler.InnerDataHandler()
        self.__ring_buffer = ring_buffer.TimeoutRingbuffer(10240 * 10240, 5)
        self._connector_change_callback = connector_change_callback

    def has_done(self):
        return self.__state == self.State.DONE

    def get_tag(self):
        return self.__tag

    def get_worker_static_info(self):
        info = super(InnerWorker,self).get_worker_static_info()
        info['tag'] = self.__tag
        info['state'] = str(self.__state).split('.')[-1]

        return info



    def __sourth_interface_event(self,event):
        if self.__state == self.State.PRE_WORKING:
            self.__recv_data_to_ringbuffer(event)
            tag = self.__data_handler.handle_pre_working_response(self.__ring_buffer)
            if tag:
                self.__tag = str(tag).strip('\0')
                self.__state = self.State.WORKING
                logger.info("InnerWorker %d trans into WORKING state,tag is:%s"%(self._worker_id,tag))
        if self.__state == self.State.WORKING:
            self.__recv_data_to_ringbuffer(event)


    def __recv_data_to_ringbuffer(self,event):
        error_happen = False
        if event.fd_event & select.EPOLLIN:
            recv_msg = self._connector.recv()
            if len(recv_msg) > 0:
                self.__ring_buffer.put(bytearray(recv_msg))
                self._recv_flow_static.add_flow(len(recv_msg))
            else:
                if self._connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                    logger.error("InnerWorker %d current state:WORKING recv data error" % (self._worker_id))
        elif event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self._connector.close()
            self.__state = self.State.CLOSED
            logger.debug("InnerWorker %d current state:WORKING change state to CLOSED" % (self._worker_id))

    def __north_interface_transdata_event(self,event):
        if not isinstance(event,forward_event.TransDataEvent):
            return
        f_data = event.forward_data
        if event.forward_data.data_type == forward_data.DATA_TYPE.NEW_CONNECTION:
            self.__data_handler.create_connection(f_data.id,f_data.inner_ip,
                                                  f_data.inner_port,self._connector)
        elif event.forward_data.data_type == forward_data.DATA_TYPE.TRANS_DATA:

            self.__data_handler.trans_data(f_data.id,f_data.inner_ip,f_data.inner_port,f_data.data,self._connector)
            self._send_flow_static.add_flow(len(f_data.data))

    def __north_interface_closecon_event(self,event):
        if self.__state in (self.State.DONE,self.State.CLOSED):
            return
        if not isinstance(event,forward_event.CloseConEvent):
            return

        if event.forward_id == self._worker_id:
            self._connector.close()
            self.__state = self.State.CLOSED
            logger.debug("InnerWorker %d current state:WORKING change state to CLOSED" % (self._worker_id))
        else:
            self.__data_handler.close_connection(event.forward_id,'0.0.0.0',0,self._connector)

    def __scheduler_event(self,event):
        if self.__state == self.State.NONE:
            self.__data_handler.send_pre_working(self._connector)
            self.__state = self.State.PRE_WORKING
            self.__pre_working_start = time.time()
            logger.debug("InnerWorker %d current state:NONE change state to PRE_WORKING" % (self._worker_id))
        elif self.__state == self.State.PRE_WORKING:
            if (time.time() - self.__pre_working_start) > 5:
                self.__state = self.State.CLOSED
                logger.error("InnerWorker %d stay in PRO_WORKING state over 5 seconds,close it"%(self._worker_id))
        elif self.__state == self.State.CLOSED:
            # close all paired outer worker
            try:
                close_event = forward_event.CloseConEvent(self._worker_id)
                self.__north_interface_channel(close_event)
                self._connector.close()
                self._connector_change_callback(self._connector, self.handler_event)
            except Exception,e:
                logger.error(e.message)
            self.__state = self.State.DONE
            logger.debug("InnerWorker %d current state:CLOSED change state to DONE" % (self._worker_id))
        elif self.__state == self.State.WORKING:
            if self._connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.CLOSED
                logger.debug("InnerWorker %d current state:WORKING change state to CLOSED due connector state error:%s"%(self._worker_id,str(self._connector.con_state)) )
                return

        self.__handle_data()

    def __handle_data(self):
        datas = self.__data_handler.get_forward_datas(self.__ring_buffer)
        for data in datas:
            if data.data_type in (forward_data.DATA_TYPE.CONNECTION_SUCCESS,
                                  forward_data.DATA_TYPE.TRANS_DATA):
                trans_event = forward_event.TransDataEvent(data.id,data)
                self.__north_interface_channel(trans_event)
            elif data.data_type == forward_data.DATA_TYPE.CLOSE_CONNECTION:
                close_event = forward_event.CloseConEvent(data.id)
                self.__north_interface_channel(close_event)
            elif data.data_type == forward_data.DATA_TYPE.HEART_BEAT:
                pass

    @forward_event.event_filter
    def handler_event(self,event):
        if event.event_type == forward_event.FDEVENT:
            # socket receive msg
            self.__sourth_interface_event(event)
        elif event.event_type == forward_event.TRANSDATAEVENT:
            self.__north_interface_transdata_event(event)
        elif event.event_type == forward_event.CLOSECONEVENT:
            self.__north_interface_closecon_event(event)
        elif event.event_type == forward_event.SCHEDULEREVENT:
            self.__scheduler_event(event)






class InfoCollection(object):

    INSTANCE = None

    @classmethod
    def get_instance(cls):
        if not InfoCollection.INSTANCE:
            InfoCollection.INSTANCE = cls()
        return InfoCollection.INSTANCE

    def set_port_mapper(self,mapper):
        self.__port_mapper = mapper

    def set_worker_manager(self,mamaner):
        self.__worker_manager = mamaner

    def get_port_mapper(self):
        ports = self.__port_mapper.get_outer_ports()

        result = {}
        for port in ports:
            ip,inner_port,tag = self.__port_mapper.get_inner_info_by_out_port(port)
            result[port] = {"ip":ip,"port":inner_port,"tag":tag}
        return result

    def create_new_port(self,port,mapper_ip,mapper_port,mapper_tag):
        self.__port_mapper.create_new_port(port,mapper_ip,mapper_port,mapper_tag)

    def delete_port(self,port):
        self.__port_mapper.delete_port(port)

    def get_inner_worker_info(self,worker_id = None):
        return self.__worker_manager.get_inner_worker_info(worker_id)

    def get_outer_worker_info(self,worker_id):
        return self.__worker_manager.get_outer_worker_info(worker_id)

    def close_outer_worker(self,worker_id):
        return self.__worker_manager.close_outer_worker(worker_id)

    def close_inner_worker(self,worker_id):
        return self.__worker_manager.close_inner_worker(worker_id)

    def get_channel_info_by_port(self,port):

        if not port:
            ports = self.__port_mapper.get_outer_ports()
        else:
            ports = [port]

        result = {}
        for p in ports:

            workers = self.__worker_manager.get_workers_by_port(p)

            p_list = []
            for worker in workers:
                # info = {}
                # info['channel_id'] = worker.get_worker_id()
                # address = worker.get_address()
                # info['ip'] = address[0]
                # info['port'] = address[1]
                # info['state'] = worker.get_state_str()
                info = worker.get_worker_static_info()
                p_list.append(info)
                result[p] = p_list
        return result





import socket
from binascii import hexlify
from pprint import pprint

from common import forward_data


class ParseTimeout(Exception):
    def __init__(self, package_len, buf_len):
        self.package_len = package_len
        self.buf_len = buf_len

    @property
    def message(self):
        return "Parse time out,package len:%d buf_len:%d"%(self.package_len,self.buf_len)

class ParseError(Exception):
    pass



class ProtocolHandler(object):


    def __init__(self):
        pass

    def print_buf(self,buf_array):
        print ' '.join(['0x%.2x'% x for x in buf_array ])
    def ip_to_bytearray(self,ip):
        packed_ip_addr = socket.inet_aton(ip)
        hexStr = hexlify(packed_ip_addr)
        result_array = bytearray(4)
        result_array[0] = int(hexStr[0:2],16)
        result_array[1] = int(hexStr[2:4],16)
        result_array[2] = int(hexStr[4:6],16)
        result_array[3] = int(hexStr[6:8],16)

        return result_array

    def bytearray_to_ip(self,ip_array):

        array2ip = lambda x:'.'.join([ str(x[i]) for i in range(4) ])

        return array2ip(ip_array)



    def build_data(self,data):
        trans_type = data.data_type
        trans_id = data.id
        inner_ip = data.inner_ip
        inner_port = data.inner_port
        msg = data.data


        msg_len = len(msg)

        buf_len = 19+msg_len

        result_buf = bytearray(buf_len)

        result_buf[0] = 0xAB
        result_buf[1] = 0xBA

        result_buf[2] = trans_type

        result_buf[3] = trans_id%0x100
        result_buf[4] = trans_id/0x100%0x100
        result_buf[5] = trans_id/0x10000%0x100
        result_buf[6] = trans_id/0x1000000

        ip_array = self.ip_to_bytearray(inner_ip)
        result_buf[7] = ip_array[0]
        result_buf[8] = ip_array[1]
        result_buf[9] = ip_array[2]
        result_buf[10] = ip_array[3]

        result_buf[11] = inner_port%0x100
        result_buf[12] = inner_port/0x100

        result_buf[13] = msg_len%0x100
        result_buf[14] = msg_len/0x100%0x100
        result_buf[15] = msg_len/0x10000%0x100
        result_buf[16] = msg_len/0x1000000

        #for i in range(msg_len):
        #    result_buf[17 + i] = msg[i]
        result_buf[17:17 + msg_len] = msg

        result_buf[buf_len - 1] = 0x16

        return result_buf

    def parse_data(self,array_buf):
        if array_buf[0] != 0xAB or array_buf[1] != 0xBA:
            return None

        result_data = forward_data.ForwardData()
        result_data.data_type = array_buf[2]
        result_data.id = array_buf[3] + array_buf[4]*0x100 + array_buf[5]*0x10000 + array_buf[6]*0x1000000


        result_data.inner_ip = self.bytearray_to_ip(array_buf[7:11])

        result_data.inner_port = array_buf[11] + array_buf[12]*0x100

        data_len = array_buf[13] + array_buf[14]*0x100 + array_buf[15]*0x10000 + array_buf[16]*0x1000000
        if data_len > 0:
            result_data.data = array_buf[17:17+data_len]

        return result_data

    def del_wrong_header(self,ring_buffer):
        i = 0;
        while True:
            if i >= ring_buffer.buf_len():
                ring_buffer.get(ring_buffer.buf_len())
                break
            if ring_buffer.look(i) != 0xAB:
                i += 1
                continue
            else:
                if ring_buffer.buf_len() < 2:
                    ring_buffer.get(ring_buffer.buf_len())
                    break
                if ring_buffer.look(i+1) != 0xBA:
                    i += 2
                    continue
                ring_buffer.get(i)
                break

    def pop_header(self,ring_buffer):
        ring_buffer.get(2)

    def get_one_complete_package(self,ring_buffer):


        if ring_buffer.buf_len() < 19:
            raise ParseTimeout(19,ring_buffer.buf_len())

        if ring_buffer.look(0) != 0xAB or ring_buffer.look(1) != 0xBA:
            raise ParseError()

        data_len = ring_buffer.look(13) + ring_buffer.look(14)*0x100 + ring_buffer.look(15)*0x10000 + ring_buffer.look(16)*0x1000000

        package_len = 19 + data_len

        if ring_buffer.buf_len() < package_len:
            raise ParseTimeout(package_len, ring_buffer.buf_len())

        if ring_buffer.look(package_len - 1) != 0x16:
            raise ParseError()

        package = ring_buffer.get(package_len)

        return package




if __name__=='__main__':
    p = ProtocolHandler()

    data = 'abcdefg'

    test_buf = p.build_data(12, forward_data.DATA_TYPE.TRANS_DATA, '192.168.1.1', 1234, data)
    p.print_buf(test_buf)
    pprint(vars(test_buf))
    pars_result = p.parse_data(test_buf)
    pprint(vars(test_buf))


    #p.print_buf(p.build_data(13,forward_data.DATA_TYPE.CLOSE_CONNECTION,'127.0.0.1',4321,''))



import outer_worker
import inner_worker
import select
from common import connector
from common import forward_event
from common import forward_data

class WorkerManager(object):
    def __init__(self,epoll_recver):
        self.__inner_map = {}
        self.__epoll_recver = epoll_recver

    def add_outer_worker(self,outer_ip, outer_port):
        self.__outer_worker = outer_worker.OuterWorker(outer_ip,outer_port,self.__outer_to_inner_channel(),self.__connector_changed)
        return self.__outer_worker

    def add_inner_worker(self,forward_id,inner_ip,inner_port):
        #forward_id = worker.get_forward_id()
        worker = inner_worker.InnerWorker(forward_id,inner_ip,inner_port,self.__inner_to_outer_channel(forward_id),self.__connector_changed)
        self.__inner_map[forward_id] = worker
        return worker

    def del_inner_worker(self,forward_id):
        self.__inner_map.pop(forward_id)

    def get_worker(self,forward_id):
        if self.__inner_map.has_key(forward_id):
            return self.__inner_map[forward_id]
        else:
            return None

    def all_do(self):
        scheduler_event = forward_event.SchedulerEvent()
        self.__outer_worker.handler_event(scheduler_event)
        for key,value in self.__inner_map.items():
            if value.has_done():
                self.__inner_map.pop(key)
                continue
            value.handler_event(scheduler_event)



    def __connector_changed(self,con,event_handler):
        if con.con_state == connector.CON_STATE.CON_CONNECTED:
            self.__epoll_recver.add_receiver(con.get_fileno(), select.EPOLLIN,event_handler)
        elif con.con_state == connector.CON_STATE.CON_CONNECTING:
            self.__epoll_recver.add_receiver(con.get_fileno(), select.EPOLLOUT,event_handler)
        elif con.con_state == connector.CON_STATE.CON_CLOSED:
            self.__epoll_recver.del_receiver(con.get_fileno())

    def close_all(self):
        for key, value in self.__inner_map.items():
            value.close()

    def __outer_to_inner_channel(self):
        def channel(event):
            if event.event_type == forward_event.TRANSDATAEVENT and \
                event.forward_data.data_type == forward_data.DATA_TYPE.NEW_CONNECTION:
                    self.add_inner_worker(event.forward_data.id,event.forward_data.inner_ip,event.forward_data.inner_port)
            elif event.event_type in (forward_event.TRANSDATAEVENT,forward_event.CLOSECONEVENT):
                if self.__inner_map.has_key(event.forward_id):
                    self.__inner_map[event.forward_id].handler_event(event)

        return channel

    def __inner_to_outer_channel(self,forward_id):
        def channel(event):
            if self.__outer_worker:
                self.__outer_worker.handler_event(event)
        return channel


import os
import sys
import json

PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '..'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
from oslo_config import cfg
from webob import Request
from webob import Response
from token_verify import token_verify

class RestApiHandler(object):

    @token_verify
    def __call__(self, environ,start_response):
        start_response("200 OK", [("Content-type", "text/plain")])
        req = Request(environ)
        res = Response()
        res.headers['Content-Type'] = 'application/json'
        params = self._get_params(req)
        if req.method == 'GET':
            self._get(req,res)
        elif req.method == 'POST':
            if params.has_key('_method') and params['_method'] == 'PUT':
                self._put(req, res)
            elif params.has_key('_method') and params['_method'] == 'DELETE':
                self._delete(req, res)
            else:
                self._post(req,res)
        elif req.method == 'PUT':
            self._put(req,res)
        elif req.method == 'DELETE':
            self._delete(req,res)
        return res(environ, start_response)

    def _get(self,request,response):
        return response

    def _post(self,request,response):
        return response

    def _put(self,request,response):
        return response

    def _delete(self,request,response):
        return response

    def _get_params(self,request):
        parms = {}
        for key,value in request.params.items():
            if type(key) == unicode:
                key = unicode.encode(key)
            if type(value) == unicode:
                value = unicode.encode(value)
            parms[key] = value
        return parms
class ClientPort(RestApiHandler):

    def _get(self,request,response):
        result = {"client_port": cfg.CONF.INNER_PORT}
        response.body = json.dumps(result)

    @classmethod
    def factory(cls,global_conf,**kwargs):
        return cls()





import os
import sys

from oslo_config import cfg

PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '..'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
import select
import logging
import traceback
from logging.handlers import RotatingFileHandler
from common import epoll_recever
from common import acceptor
import port_mapper
import conf
import worker_manager
from rest_api import info_collection

logger = logging.getLogger('my_logger')

port_mapper_changed = False


def run():
    recver = epoll_recever.Epoll_receiver()

    inner_port = int(cfg.CONF.INNER_PORT)
    inner_acceptor = acceptor.Acceptor('0.0.0.0', inner_port)
    logger.info('Inner port listen:' + str(inner_port))

    out_acceptors = {}

    # there are many outer acceptor





    def build_outer_acceptors2(new_ports = [],deld_ports = [],changed_ports = []):
        for port in deld_ports+changed_ports:
            recver.del_receiver(out_acceptors[port].get_fileno())
            out_acceptors[port].close()
            out_acceptors.pop(port)
            logger.info("Out port deled:" + str(port))

        for port in new_ports+changed_ports:
            _inner_ip, _inner_port, _inner_tag = _port_mapper.get_inner_info_by_out_port(port)
            _outer_acceptor = acceptor.PortMapperAcceptor('0.0.0.0', port, _inner_ip, _inner_port, _inner_tag)
            recver.add_receiver(_outer_acceptor.get_fileno(), select.EPOLLIN,
                                outer_acceptor_handler_event(_outer_acceptor))
            out_acceptors[port] = _outer_acceptor
            logger.info("Out port listen:" + str(port) + '->' + str((_inner_ip, _inner_port, _inner_tag)))

    def inner_acceptor_handler_event(event):
        _inner_socket, address = inner_acceptor.accept()
        _inner_socket.setblocking(0)
        _inner_worker = _worker_manager.add_inner_worker(_inner_socket,address)

        recver.add_receiver(_inner_socket.fileno(), select.EPOLLIN,_inner_worker.handler_event)
        logger.info('inner worker accept fileno:' + str(_inner_socket.fileno()))

    def outer_acceptor_handler_event(acceptor):
        def handler(event):
            _outer_socket, address = acceptor.accept()
            _outer_socket.setblocking(0)
            try:
                _outer_worker = _worker_manager.add_outer_worker(_outer_socket,acceptor.get_listen_port(),
                                                                 address,acceptor.mapper_ip,acceptor.mapper_port,
                                                                 acceptor.mapper_tag)
                recver.add_receiver(_outer_socket.fileno(), select.EPOLLIN,_outer_worker.handler_event)
                logger.info("Outer socket accept, worker_id:%d fileno:%d to inner port:%s:%d" %
                            (_outer_worker.get_worker_id(), _outer_socket.fileno(), acceptor.mapper_ip, acceptor.mapper_port))
            except Exception,e:
                _outer_socket.close()
                logger.error(e.message)
                logger.debug(traceback.format_exc())
        return handler

    def port_mapper_change_callback(new_ports = [],deld_ports = [],changed_ports = []):
        build_outer_acceptors2(new_ports,deld_ports,changed_ports)

    _worker_manager = worker_manager.WorkerManager(recver)

    #_port_mapper = port_mapper.FileMonitorPortMapper(cfg.CONF.OUTER_PORTS_FILE, build_outer_acceptors2)
    _port_mapper = port_mapper.InterfaceDriverPortMapper(cfg.CONF.OUTER_PORTS_FILE, build_outer_acceptors2)

    recver.add_receiver(inner_acceptor.get_fileno(), select.EPOLLIN, inner_acceptor_handler_event)

    build_outer_acceptors2(new_ports=_port_mapper.get_outer_ports())

    info_collection.InfoCollection.get_instance().set_port_mapper(_port_mapper)
    info_collection.InfoCollection.get_instance().set_worker_manager(_worker_manager)

    while True:
        #_port_mapper.watch_event(0.1)
        _worker_manager.all_worker_do()
        recver.run()

def log_config(level):
    if not os.path.isdir('/var/log/tcp_forward'):
        os.makedirs('/var/log/tcp_forward')
    logger.setLevel(logging._levelNames[level])
    handler = RotatingFileHandler("/var/log/tcp_forward/forward_server.log", maxBytes=10000000, backupCount=10)
    console = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s %(process)d %(levelname)s %(filename)s:%(lineno)s %(funcName)s [-] %(message)s ')
    handler.setFormatter(formatter)
    console.setFormatter(formatter)
    logger.addHandler(handler)
    logger.addHandler(console)


def main():
    conf.register_core_common_config_opts(cfg.CONF)

    log_config(cfg.CONF.LOG_LEVEL)

    logger.info("Process start!")
    run()

if __name__ == '__main__':
    cfg.CONF(sys.argv[1:])
    sys.exit(main())


from oslo_config import cfg

core_opts = [
    cfg.StrOpt('SERVER_IP',default='127.0.0.1',help='Server IP'),
    cfg.IntOpt('SERVER_PORT',default=1111,help='The port the server is listening on'),
    cfg.StrOpt('LOG_LEVEL',default='INFO',help='log level:CRITICAL ERROR WARNING INFO DEBUG NOTSET'),
    cfg.StrOpt('TAG',default='default',help='the client tag')
]


def register_core_common_config_opts(cfg=cfg.CONF):
    cfg.register_opts(core_opts)


from oslo_config import cfg

core_opts = [
    cfg.IntOpt('INNER_PORT',default=1111,help='port which client to connect'),
    cfg.StrOpt('OUTER_PORTS_FILE',default='/etc/tcp-forward/port_mapper.cfg',help='port mapper file'),
    cfg.StrOpt('LOG_LEVEL',default='INFO',help='log level:CRITICAL ERROR WARNING INFO DEBUG NOTSET'),
    cfg.StrOpt('BANDWIDTH',default='100',help='the max network bandwidth,use M as unit'),
    cfg.StrOpt('user_file',default='/etc/tcp-forward/user_file',help='user file'),
]


def register_core_common_config_opts(cfg=cfg.CONF):
    cfg.register_opts(core_opts)


import api
import info_collection
import json

class ClientConnectionApi(api.RestApiHandler):

    def _get(self,request,response):
        params = self._get_params(request)

        if params.has_key('client_id'):
            worker_id = params['client_id']
        else:
            worker_id = None
        collection = info_collection.InfoCollection.get_instance()

        response.body = json.dumps(collection.get_inner_worker_info(worker_id))

    def _delete(self,request,response):
        params = self._get_params(request)

        try:
            if params.has_key('client_id'):
                info_collection.InfoCollection.get_instance().close_inner_worker(int(params['client_id']))
            elif params.has_key('client_ids'):
                clients_list = json.loads(params['client_ids'])
                for client_id in clients_list:
                    info_collection.InfoCollection.get_instance().close_inner_worker(int(client_id))
        except Exception, e:
            response.body = str({'result': 'failed', 'reason': e.message})
            return

        response.body = json.dumps({'result': 'success'})

    @classmethod
    def factory(cls, global_conf, **kwargs):
        return cls()


from webob import Request
from webob import Response
from itsdangerous import TimedJSONWebSignatureSerializer as Serializer
from itsdangerous import SignatureExpired
from itsdangerous import BadSignature

ENCRYPT_KEY = 'TCP-FORWARD'


def get_token(data):
    # expires_in is token timeout seconds
    s = Serializer(ENCRYPT_KEY, expires_in=3600)


    return s.dumps(data)



def verify_auth_token(token):
    s = Serializer(ENCRYPT_KEY)
    try:
        data = s.loads(token)
    except SignatureExpired:
        return None  # valid token, but expired
    except BadSignature:
        return None  # invalid token

    return data

def token_verify(func):
    def verity(*args,**kwargs):
        environ = args[1]
        start_response = args[2]

        req = Request(environ)


        if req.url.split('/')[-1] == 'token':
            return func(*args,**kwargs)


        token = req.cookies.get('X-Auth-Token') or req.headers.get('X-Auth-Token')

        def unauthorized_reply(environ,start_response):
            res = Response()
            res.status = '401 Unauthorized'
            res.body = 'Unauthorized'
            res.headers.add('Www-Authenticate ', '/'.join(x for x in (req.url.split('/')[:-1] + ['token'])))
            return res(environ, start_response)

        if not token:
            return unauthorized_reply(environ,start_response)

        data = verify_auth_token(token)

        if not data:
            return unauthorized_reply(environ,start_response)

        return func(*args,**kwargs)

    return verity


from enum import Enum
import logging
import time
import select
import traceback
from common import connector
from common import forward_event
from common import forward_data
import worker

logger = logging.getLogger('my_logger')

import data_handler
class OuterWorker(worker.Worker):
    class State(Enum):
        NONE = 0
        CONNECTING_TO_INNER = 1
        WORKING = 2
        CLOSED = 3
        DONE = 4

    def __init__(self,worker_id,inner_ip,inner_port,outer_socket,address,sourth_interface_channel,connector_change_callback):
        super(OuterWorker, self).__init__(worker_id, address, connector.Connector(outer_socket))
        self.__inner_ip = inner_ip
        self.__inner_port = inner_port
        self.__state = self.State.NONE
        self.__sourth_interface_channel = sourth_interface_channel
        self.__data_handler = data_handler.OutDataHandler()
        self._connector_change_callback = connector_change_callback
        self.__connecting_begin_time = 0

    def has_done(self):
        return self.__state == self.State.DONE

    def get_state_str(self):
        return str(self.__state).split('.')[-1]

    def get_worker_static_info(self):
        info = super(OuterWorker,self).get_worker_static_info()
        info['state'] = str(self.__state).split('.')[-1]

        return info


    def __recv_data(self,event):
        error_happen = False
        if event.fd_event & select.EPOLLIN:
            recv_msg = self._connector.recv()
            if len(recv_msg) > 0:
                # trans data
                # print 'recv outer %d msg:'%(self._worker_id)
                # tools.print_hex_buf(recv_msg)
                try:
                    trans_data = forward_data.ForwardData(forward_data.DATA_TYPE.TRANS_DATA,self._worker_id,self.__inner_ip,self.__inner_port,recv_msg)
                    trans_data_event = forward_event.TransDataEvent(self._worker_id,trans_data)

                    self.__sourth_interface_channel(trans_data_event)
                    self._recv_flow_static.add_flow(len(recv_msg))
                except Exception, e:
                    error_happen = True
                    logger.error("OuterWorker %d current state:WORKING send data error" % (self._worker_id))
                    logger.debug(traceback.format_exc())
            else:
                if self._connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                logger.error("OuterWorker %d current state:WORKING recv data error" % (self._worker_id))

        elif event.fd_event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self._connector.close()

            self.__state = self.State.CLOSED
            logger.debug("OuterWorker %d current state:WORKING change state to CLOSED" % (self._worker_id))


    def __north_interface_event(self, event):
        if self.__state == self.State.WORKING:
            self.__recv_data(event)
        else:
            time.sleep(0.01)

    def __sourth_interface_transdata_event(self,event):
        if not isinstance(event,forward_event.TransDataEvent):
            return

        if self.__state == self.State.CONNECTING_TO_INNER:
            if event.forward_data.data_type == forward_data.DATA_TYPE.CONNECTION_SUCCESS:
                self.__state = self.State.WORKING
                logger.debug(
                    "OuterWorker %d current state:CONNECTING_TO_INNER change state to WORKING" % (self._worker_id))
        elif self.__state == self.State.WORKING:
            if event.forward_data.data_type == forward_data.DATA_TYPE.TRANS_DATA:
                send_bytes = self._connector.send(event.forward_data.data)
                if send_bytes <= 0:
                    logger.error("OutWorker %d trans bytes <=0 change state to CLOSED", self._worker_id)
                    self.__state = self.State.CLOSED
                else:
                    self._send_flow_static.add_flow(send_bytes)
        else:
            pass


    def __sourth_interface_closecon_event(self,event):
        if self.__state in (self.State.DONE,self.State.CLOSED):
            return

        self._connector.close()
        self.__state = self.State.CLOSED
        logger.debug(
            "OuterWorker %d current state:%s change state to CLOSED " % (self._worker_id, str(self.__state)))

    def __scheduler_event(self, event):
        if self.__state == self.State.NONE:
            self.__data_handler.create_connection(self._worker_id, self.__inner_ip, self.__inner_port,
                                                  self.__sourth_interface_channel)
            self.__state = self.State.CONNECTING_TO_INNER
            self.__connecting_begin_time = time.time()
            logger.debug("OuterWorker %d current state:NONE create inner connecttion to %s:%d ,change state to CONNECTING_TO_INNER" %
                (self._worker_id, self.__inner_ip, self.__inner_port))
        elif self.__state == self.State.CONNECTING_TO_INNER:
            if (time.time() - self.__connecting_begin_time) >= 5:
                #connecting can not over 5 seconds
                self.__state = self.State.CLOSED
                logger.error("OuterWorker %d  stay in CONNECTING_TO_INNER state over 5 seconds,change state to CLOSED"%(self._worker_id))
        elif self.__state == self.State.WORKING:
            if self._connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.CLOSED
                logger.debug("OuterWorker %d current state:WORKING change state to CLOSED due connector state error:%s"%(self._worker_id,str(self._connector.con_state)) )
        elif self.__state == self.State.CLOSED:
            # send closed msg to remote
            close_event = forward_event.CloseConEvent(self._worker_id)
            self.__sourth_interface_channel(close_event)
            self._connector.close()
            self._connector_change_callback(self._connector, self.handler_event)
            self.__state = self.State.DONE
            logger.debug("OuterWorker %d current state:CLOSED change state to DONE" % (self._worker_id))

    @forward_event.event_filter
    def handler_event(self,event):
        if event.event_type == forward_event.FDEVENT:
            # socket receive msg
            self.__north_interface_event(event)
        elif event.event_type == forward_event.TRANSDATAEVENT:
            self.__sourth_interface_transdata_event(event)
        elif event.event_type == forward_event.CLOSECONEVENT:
            self.__sourth_interface_closecon_event(event)
        elif event.event_type == forward_event.SCHEDULEREVENT:
            self.__scheduler_event(event)


from oslo_config import cfg
from common import forward_data, protocol_handler
from common import connector
from common.data_handler import DataHandler
import logging
logger = logging.getLogger('my_logger')

class OuterDataHandler(DataHandler):
    def __init__(self):
        self.__one_package_size = 2**31

    def send_preworking_reply(self,outer_connector):
        tag = cfg.CONF.TAG
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.PRE_WORKING, 0, '0.0.0.0', 0, tag)
        protocol_parser = protocol_handler.ProtocolHandler()
        send_package = protocol_parser.build_data(forw_data)
        outer_connector.send(send_package)

    def inner_connect_succed(self, forward_id, inner_ip, inner_port, outer_connector):
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.CONNECTION_SUCCESS, forward_id, inner_ip, inner_port)
        protocol_parser = protocol_handler.ProtocolHandler()
        send_package = protocol_parser.build_data(forw_data)
        outer_connector.send(send_package)

    def close_connection(self,forward_id,inner_ip,inner_port,outer_connector):
        _protocol_handler = protocol_handler.ProtocolHandler()
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.CLOSE_CONNECTION, forward_id,
                                             inner_ip, inner_port, '')
        send_package = _protocol_handler.build_data(forw_data)
        send_bytes = outer_connector.send(send_package)
        if send_bytes <= 0:
            logger.error("CloseConnectionData send failed,forward_id:%d inner_ip:%s inner_port:%d" % (forward_id, inner_ip, inner_port))

    def trans_data(self,forward_id,inner_ip,inner_port,data,outer_connector):
        ori = 0
        total_len = len(data)

        while ori < total_len:
            if total_len - ori <= self.__one_package_size:
                send_data = data[ori:total_len]
            else:
                send_data = data[ori:ori + self.__one_package_size]

            _protocol_handler = protocol_handler.ProtocolHandler()
            forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.TRANS_DATA, forward_id,
                                                 inner_ip, inner_port, send_data)
            send_package = _protocol_handler.build_data(forw_data)
            if outer_connector and outer_connector.con_state == connector.CON_STATE.CON_CONNECTED:
                send_bytes = outer_connector.send(send_package)
                if send_bytes <= 0:
                    logger.error("TransData to inner send failed,forward_id:%d inner_ip:%s inner_port:%d" % (
                    forward_id, inner_ip, inner_port))
                    raise Exception("Send data failed")
                    # print 'inner_connector send package'
                    # tools.print_hex_buf(send_package)
            ori += self.__one_package_size

class InnerDataHandler(DataHandler):
    pass



FDEVENT = 'FDEvent'
TRANSDATAEVENT = 'TransDataEvent'
CLOSECONEVENT = 'CloseConEvent'
SCHEDULEREVENT = 'SchedulerEvent'

class ForwardEvent(object):
    def __init__(self,event_type):
        self.event_type = event_type

class FDEvent(ForwardEvent):
    def __init__(self,fd_event):
        super(FDEvent,self).__init__(FDEVENT)
        self.fd_event = fd_event

class SchedulerEvent(ForwardEvent):
    def __init__(self):
        super(SchedulerEvent,self).__init__(SCHEDULEREVENT)

class TransDataEvent(ForwardEvent):
    def __init__(self,forward_id,forward_data):
        super(TransDataEvent,self).__init__(TRANSDATAEVENT)
        self.forward_id = forward_id
        self.forward_data = forward_data

class CloseConEvent(ForwardEvent):
    def __init__(self,forward_id):
        super(CloseConEvent,self).__init__(CLOSECONEVENT)
        self.forward_id = forward_id


def event_filter(func):
    def filter(*args,**kwargs):
        if type(args[1]) == int:
            f_event = FDEvent(args[1])
        else:
            f_event = args[1]

        if not isinstance(f_event,ForwardEvent):
            raise Exception('event_filter event is not subclass of ForwardEvent:' + str(type(f_event)))

        return func(args[0],f_event,*args[2:],**kwargs)
    return filter


import random
import select
import inner_worker
import outer_worker
from common import forward_event
from common import connector

class WorkerManager(object):
    def __init__(self,epoll_recver):
        self.__mapper = {}
        self.__outer_workers = {}
        self.__inner_workers = {}
        self.__worker_id_to_port = {}
        self.__worker_id_seq = 0
        self.__epoll_recver = epoll_recver

    def generate_worker_id(self):
        self.__worker_id_seq += 1
        return self.__worker_id_seq

    def add_map(self,outer_worker_id,inner_worker_id):
        self.__mapper[outer_worker_id] = inner_worker_id

    def add_outer_worker(self,outer_socket,port,address,inner_ip,inner_port,inner_tag):
        _worker_id = self.generate_worker_id()
        _paired_inner_worker = self.get_inner_worker_by_tag(inner_tag)


        if not _paired_inner_worker :
            raise Exception('get inner worker by tag failed,tag is:' + str(inner_tag))
        _outer_worker = outer_worker.OuterWorker(_worker_id, inner_ip,inner_port, outer_socket,address,
                                                 self.__outer_to_inner_channel(_worker_id,_paired_inner_worker.get_worker_id()),
                                                 self.__connector_changed)

        self.add_map(_worker_id, _paired_inner_worker.get_worker_id())
        self.__outer_workers[_worker_id] = _outer_worker
        self.__worker_id_to_port[_worker_id] = port
        return _outer_worker

    def add_inner_worker(self,inner_socket,address):

        _worker_id = self.generate_worker_id()
        _inner_worker = inner_worker.InnerWorker(_worker_id, inner_socket, address,self.__inner_to_outer_channel(_worker_id),
                                                 self.__connector_changed)

        self.__inner_workers[_worker_id] = _inner_worker
        return _inner_worker

    def get_worker_by_id(self,worker_id):
        if self.__outer_workers.has_key(worker_id):
            return self.__outer_workers[worker_id]
        elif self.__inner_workers.has_key(worker_id):
            return self.__inner_workers[worker_id]
        else:
            return None

    def get_workers_by_port(self,port):
        workers = []
        for worker_id,o_port in self.__worker_id_to_port.items():
            if o_port == port:
                workers.append(self.__outer_workers[worker_id])
        return workers

    def get_inner_worker_by_random(self):
        if len(self.__inner_workers) <= 0:
            return None

        _random_num = int(random.random()*100)%len(self.__inner_workers)
        return [value for value in self.__inner_workers.values()][_random_num]

    def get_inner_worker_by_tag(self,tag):
        if len(self.__inner_workers) <= 0:
            return None
        for item in self.__inner_workers.values():
            if item.get_tag() == tag:
                return item
        return None

    def remove_outer_worker(self,worker_id):
        self.__outer_workers.pop(worker_id)
        if self.__mapper.has_key(worker_id):
            self.__mapper.pop(worker_id)
        self.__worker_id_to_port.pop(worker_id)

    def remove_inner_worker(self,worker_id):
        self.__inner_workers.pop(worker_id)
        for key,value in self.__mapper.items():
            if value == worker_id:
                self.__mapper.pop(key)
    def all_worker_do(self):
        scheduler_event = forward_event.SchedulerEvent()
        for id,worker in self.__inner_workers.items():
            if worker.has_done():
                self.remove_inner_worker(id)
                continue
            worker.handler_event(scheduler_event)

        for id,worker in self.__outer_workers.items():
            if worker.has_done():
                self.remove_outer_worker(id)
                continue
            worker.handler_event(scheduler_event)

    def get_inner_worker_info(self,worker_id = None):
        info = {}
        if worker_id:
            if self.__inner_workers.has_key(worker_id):
                info[worker_id] = self.__inner_workers[worker_id].get_worker_static_info()
        else:
            for id,worker in self.__inner_workers.items():
                info[id] = worker.get_worker_static_info()
        return info

    def get_outer_worker_info(self,worker_id):
        info = {}
        if self.__outer_workers.has_key(worker_id):
            info[worker_id] = self.__outer_workers[worker_id].get_worker_static_info()
        return info

    def close_outer_worker(self,worker_id):
        if not self.__outer_workers.has_key(worker_id):
            raise Exception('can not find this worker:' + str(worker_id))

        close_event = forward_event.CloseConEvent(worker_id)
        self.__outer_workers[worker_id].handler_event(close_event)

    def close_inner_worker(self,worker_id):
        if not self.__inner_workers.has_key(worker_id):
            raise Exception('can not find this worker:' + str(worker_id))
        close_event = forward_event.CloseConEvent(worker_id)
        self.__inner_workers[worker_id].handler_event(close_event)


    def __connector_changed(self,con,event_handler):
        if con.con_state == connector.CON_STATE.CON_CONNECTED:
            self.__epoll_recver.add_receiver(con.get_fileno(), select.EPOLLIN,event_handler)
        elif con.con_state == connector.CON_STATE.CON_CONNECTING:
            self.__epoll_recver.add_receiver(con.get_fileno(), select.EPOLLOUT,event_handler)
        elif con.con_state == connector.CON_STATE.CON_CLOSED:
            self.__epoll_recver.del_receiver(con.get_fileno())

    def __inner_to_outer_channel(self,inner_worker_id):
        def channel(event):
            if event.event_type == forward_event.TRANSDATAEVENT:
                if not self.__mapper.has_key(event.forward_id) or self.__mapper[event.forward_id] != inner_worker_id:
                    return
                if not self.__outer_workers.has_key(event.forward_id):
                    return
                self.__outer_workers[event.forward_id].handler_event(event)
            elif event.event_type == forward_event.CLOSECONEVENT:
                if event.forward_id == inner_worker_id:
                    # innerworker closed,close all associated outerworker
                    for outer_id,inner_id in self.__mapper.items():
                        if inner_id != inner_worker_id:
                            continue
                        if not self.__outer_workers.has_key(outer_id):
                            continue
                        self.__outer_workers[outer_id].handler_event(event)
                else:
                    # just close one outer worker
                    if not self.__outer_workers.has_key(event.forward_id):
                        return
                    self.__outer_workers[event.forward_id].handler_event(event)
        return channel

    def __outer_to_inner_channel(self,outer_worker_id,inner_worker_id):
        def channel(event):
            if self.__inner_workers.has_key(inner_worker_id):
                self.__inner_workers[inner_worker_id].handler_event(event)
        return channel


import select

from enum import Enum

import data_handler
from common import connector
from common import forward_event
from common import forward_data
import logging
import traceback
import data_handler
logger = logging.getLogger('my_logger')

class InnerWorker(object):
    class State(Enum):
        NONE = 0
        CONNECTING = 1
        WORKING = 2
        CLOSED = 3
        DONE = 4

    def __init__(self,forward_id,inner_ip,inner_port,north_interface_channel,connector_change_callback):
        self.__state = self.State.NONE
        self.__id = forward_id
        self.__inner_ip = inner_ip
        self.__inner_port = inner_port
        self.__connector = None
        self.__data_handler = data_handler.InnerDataHandler()
        self.__north_interface_channel = north_interface_channel
        self.__connector_change_callback = connector_change_callback

    def has_done(self):
        return self.__state == self.State.DONE

    def __sourth_interface_event(self,event):
        if self.__state == self.State.CONNECTING:
            self.__connector.refresh_con_state()
            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                #self.__data_handler.inner_connected(self.__id, self.__inner_ip, self.__inner_port, outer_connector)
                self.__connecte_success()
                self.__state = self.State.WORKING
                logger.info("InnerWorker %d connect to %s:%d succed,change state to WORKING" % (
                self.__id, self.__inner_ip, self.__inner_port))
            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                self.__connector.close()
                self.__state = self.State.CLOSED
                logger.error("InnerWorker %d connect to %s:%d failed,change state to CLOSED" % (
                self.__id, self.__inner_ip, self.__inner_port))
        elif self.__state == self.State.WORKING:
            self.__handle_working_event(event)



    def __north_interface_transdata_event(self,event):
        if not isinstance(event,forward_event.TransDataEvent):
            return

        if self.__state == self.State.WORKING:
            if event.forward_data.data_type == forward_data.DATA_TYPE.TRANS_DATA:
                send_bytes = self.__connector.send(event.forward_data.data)
                if send_bytes <= 0:
                    logger.error("InnerWorker %d trans bytes <=0 change state to CLOSED", self.__id)
                    self.__state = self.State.CLOSED

    def __north_interface_closecon_event(self,event):
        if self.__state in (self.State.DONE,self.State.CLOSED):
            return

        self.__connector.close()
        self.__state = self.State.CLOSED
        logger.debug(
            "InnerWorker %d current state:%s change state to CLOSED " % (self.__id, str(self.__state)))

    def __connecte_success(self):
        connected_data = forward_data.ForwardData(forward_data.DATA_TYPE.CONNECTION_SUCCESS, self.__id,
                                                  self.__inner_ip, self.__inner_port, '')
        connected_event = forward_event.TransDataEvent(self.__id, connected_data)
        self.__north_interface_channel(connected_event)
        self.__connector_change_callback(self.__connector, self.handler_event)

    def __scheduler_event(self, event):

        if not isinstance(event,forward_event.SchedulerEvent):
            return

        if self.__state == self.State.NONE:
            self.__connector = connector.ConnectorClient(self.__inner_ip, self.__inner_port)
            self.__connector.connect()

            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:

                self.__state = self.State.WORKING
                self.__connecte_success()
            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                self.__state = self.State.CLOSED
            elif self.__connector.con_state == connector.CON_STATE.CON_CONNECTING:
                self.__connector_change_callback(self.__connector, self.handler_event)
                self.__state = self.State.CONNECTING
            logger.debug("InnerWorker %d connect to %s:%d current state:%s"%(self.__id,self.__inner_ip,self.__inner_port,str(self.__state)))
        elif self.__state == self.State.WORKING:
            if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.CLOSED
                logger.debug("InnerWorker %d current state:WORKING change state to DONE due connector state error:%s"%(self.__id,str(self.__connector.con_state)) )
        elif self.__state == self.State.CLOSED:
            #send closed message to outer
            #self.__data_handler.inner_closed(self.__id,self.__inner_ip,self.__inner_port,outer_connector)
            close_event = forward_event.CloseConEvent(self.__id)
            self.__north_interface_channel(close_event)
            self.__connector.close()
            self.__connector_change_callback(self.__connector, self.handler_event)
            self.__state = self.State.DONE
            logger.debug("InnerWorker %d current state:CLOSED change state to DONE"%(self.__id))

    def __handle_working_event(self, event):
        error_happen = False
        if event.fd_event & select.EPOLLIN:
            recv_msg = self.__connector.recv()
            if len(recv_msg) > 0:
                # trans data
                try:
                    trans_data = forward_data.ForwardData(forward_data.DATA_TYPE.TRANS_DATA, self.__id,
                                                          self.__inner_ip, self.__inner_port, bytearray(recv_msg))
                    trans_data_event = forward_event.TransDataEvent(self.__id, trans_data)

                    self.__north_interface_channel(trans_data_event)
                except Exception, e:
                    error_happen = True
                    logger.error("InnerWorker %d current state:WORKING send data error" % (self.__id))
                    logger.debug(traceback.format_exc())

            else:
                if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                    logger.error("InnerWorker %d current state:WORKING recv data error" % (self.__id))

        elif event.fd_event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self.__connector.close()
            self.__state = self.State.CLOSED
            logger.debug("InnerWorker %d current state:WORKING change state to CLOSED" % (self.__id))

    @forward_event.event_filter
    def handler_event(self, event):
        if event.event_type == forward_event.FDEVENT:
            # socket receive msg
            self.__sourth_interface_event(event)
        elif event.event_type == forward_event.TRANSDATAEVENT:
            self.__north_interface_transdata_event(event)
        elif event.event_type == forward_event.CLOSECONEVENT:
            self.__north_interface_closecon_event(event)
        elif event.event_type == forward_event.SCHEDULEREVENT:
            self.__scheduler_event(event)


import api
import info_collection
import json

class PortMapperApi(api.RestApiHandler):
    def _get(self,request,response):
        result =  self.__get_request()
        response.body = result

    def _post(self,request,response):
        #create new port mapper

        params = self._get_params(request)
        if not params.has_key('port') or \
            not params.has_key('mapper_ip') or \
            not params.has_key('mapper_port') or \
            not params.has_key('mapper_tag'):
            response.body = json.dumps({'result':'failed','reason':'parms error'})
            return

        try:


            info_collection.InfoCollection.get_instance().create_new_port(int(params['port']),
                                                                      params['mapper_ip'],
                                                                      int(params['mapper_port']),
                                                                      params['mapper_tag'])
        except Exception,e:
            response.body =json.dumps({'result': 'failed', 'reason': e.message})
            return

        response.body = json.dumps({'result': 'success'})

    def _delete(self,request,response):
        #delete port mapper

        params = self._get_params(request)

        if not params.has_key('port') and not params.has_key('ports'):
            return [json.dumps({'result': 'failed', 'reason': 'parms error'})]

        try:
            if params.has_key('port'):
                info_collection.InfoCollection.get_instance().delete_port(int(params['port']))
            elif params.has_key('ports'):
                ports_list = json.loads(params['ports'])
                for port in ports_list:
                    info_collection.InfoCollection.get_instance().delete_port(int(port))
        except Exception,e:
            response.body = json.dumps({'result': 'failed', 'reason': e.message})
            return
        response.body = json.dumps({'result': 'success'})

    def __get_request(self):
        collection = info_collection.InfoCollection.get_instance()
        return json.dumps(collection.get_port_mapper())


    @classmethod
    def factory(cls,global_conf,**kwargs):
        return cls()




import time

class FlowStatic(object):


    def __init__(self):
        self.__init_time = time.time()
        self.__total_flow = 0   #unit:MB
        self.__real_time_speed = 0
        self.__last_add_time = 0

    def add_flow(self,flow):
        self.__total_flow = self.__total_flow + flow
        current_time = time.time()
        if current_time - self.__last_add_time < 3:
            self.__real_time_speed = float(flow)/1024/1024/(current_time - self.__last_add_time)
        else:
            self.__real_time_speed = 0
        self.__last_add_time = current_time

    def get_real_speed(self):
        current_time = time.time()
        if current_time - self.__last_add_time >= 3:
            self.__real_time_speed = 0

        return self.__real_time_speed

    def get_average_speed(self):
        return float(self.__total_flow)/1024/1024/(time.time() - self.__init_time)

    def get_init_time(self):
        return self.__init_time

    def get_total_flow(self):
        return float(self.__total_flow)/1024/1024

    def get_duration(self):
        return time.time() - self.__init_time






import logging

from common import connector
from common import forward_data
from common import forward_event
from common.data_handler import DataHandler
from common.protocol_handler import ProtocolHandler
from oslo_config import cfg

logger = logging.getLogger('my_logger')

class OutDataHandler(DataHandler):
    """
    In this section,we must calculate an suitable package size according to the network bandwidth
    Suppose I want the network card to send a packet of data within one second,then the package size must smaller than the bandwidth
    So i calculated the approximate value by dividing the bandwidth by two
    """
    def __init__(self):
        self.__bandwidth = cfg.CONF.BANDWIDTH
        self.__one_package_size = int(self.__bandwidth) * 1024 * 1024 / 8 / 2

    def create_connection(self,forward_id,inner_ip,inner_port,sourth_interface):
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.NEW_CONNECTION, forward_id,
                                             inner_ip, inner_port, '')

        forw_event = forward_event.TransDataEvent(forward_id,forw_data)
        sourth_interface(forw_event)



class InnerDataHandler(DataHandler):
    def __init__(self):
        self.__bandwidth = cfg.CONF.BANDWIDTH
        self.__one_package_size = int(self.__bandwidth) * 1024 * 1024 / 8 / 2

    def send_pre_working(self,inner_connector):
        _protocol_handler = ProtocolHandler()
        pre_work_data = forward_data.ForwardData(forward_data.DATA_TYPE.PRE_WORKING,0,'0.0.0.0',0,'')
        send_package = _protocol_handler.build_data(pre_work_data)
        if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = inner_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("Preworking data send error")
                raise Exception("Send data failed")

    def handle_pre_working_response(self, ring_buffer):
        while True:
            parsed_data = super(InnerDataHandler, self).parse_data(ring_buffer)
            if parsed_data == None:
                return None

            if parsed_data.data_type == forward_data.DATA_TYPE.PRE_WORKING:
                tag = str(parsed_data.data)
                return tag
    def get_forward_datas(self, ring_buffer):
        forward_datas = []
        while True:
            parsed_data = super(InnerDataHandler, self).parse_data(ring_buffer)
            if parsed_data == None:
                return tuple(forward_datas)
            forward_datas.append(parsed_data)

    def create_connection(self,forward_id,inner_ip,inner_port,inner_connector):
        _protocol_handler = ProtocolHandler()
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.NEW_CONNECTION, forward_id,
                                             inner_ip, inner_port, '')
        send_package = _protocol_handler.build_data(forw_data)
        if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = inner_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("CreateConnectionData send failed,forward_id:%d inner_ip:%s inner_port:%d" % (
                forward_id, inner_ip, inner_port))


    def close_connection(self,forward_id,inner_ip,inner_port,inner_connector):
        _protocol_handler = ProtocolHandler()
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.CLOSE_CONNECTION, forward_id,
                                             inner_ip, inner_port, '')
        send_package = _protocol_handler.build_data(forw_data)
        if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = inner_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("CloseConnectionData send failed,forward_id:%d inner_ip:%s inner_port:%d" % (forward_id, inner_ip, inner_port))

    def trans_data(self,forward_id,inner_ip,inner_port,data,inner_connector):
        ori = 0
        total_len = len(data)

        while ori < total_len:
            if total_len - ori <= self.__one_package_size:
                send_data = data[ori:total_len]
            else:
                send_data = data[ori:ori + self.__one_package_size]

            _protocol_handler = ProtocolHandler()
            forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.TRANS_DATA, forward_id,
                                                 inner_ip, inner_port, send_data)
            send_package = _protocol_handler.build_data(forw_data)
            if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
                send_bytes = inner_connector.send(send_package)
                if send_bytes <= 0:
                    logger.error("TransData to inner send failed,forward_id:%d inner_ip:%s inner_port:%d" % (
                    forward_id, inner_ip, inner_port))
                    raise Exception("Send data failed")
                    # print 'inner_connector send package'
                    # tools.print_hex_buf(send_package)
            ori += self.__one_package_size


from webob import Request
from webob import Response

class Test(object):

    def __call__(self, environ, start_response):
        start_response("200 OK", [("Content-type", "text/plain")])
        # print environ
        req = Request(environ)
        print req.method
        print req.url
        print req.params
        res = Response()


        res.headers.add('X-Auth-Token','eyJhbGciOiJIUzI1NiIsImV4cCI6MTUzMDY4NzI2OCwiaWF0IjoxNTMwNjg2NjY4fQ.eyJpZCI6MTIzNDU2fQ.rwNSzsQ6oSHQ_RchsGAwiiyomlc6JMp1SeE7YDFL-iM')

        res.body = "Paste Deploy LAB: Version = 1.0.0"

        return res(environ,start_response)

    @classmethod
    def factory(cls, global_conf, **kwargs):
        return cls()



from api import RestApiHandler
import token_verify

from webob.headers import ResponseHeaders
import user_pass

class Token(RestApiHandler):



    def _post(self,request,response):
        #str to dict
        body = eval(request.body)
        if not body.has_key('user') or \
            not body.has_key('password'):
            response.status = '400 Bad Request'
            response.body = 'Please enter user and password'
            return


        user = body['user']
        password = body['password']

        up = user_pass.UserPass()
        try:
            up = user_pass.UserPass()
            up.check_pass_word(user,password)
            token = token_verify.get_token({'user': user})
            response.headers.add('X-Auth-Token', token)

            response.body = str({})
        except user_pass.NoUserException:
            response.status = '401 Unauthorized'
            response.body = 'authentication failed:no uesr'
        except user_pass.PassErrorException:
            response.status = '401 Unauthorized'
            response.body = 'authentication failed:password error'

        # if self.__verify_user(user,password):
        #     token = token_verify.get_token({'user':user})
        #     response.headers.add('X-Auth-Token',token)
        #
        #     response.body = str({})
        # else:
        #     response.status = '401 Unauthorized'
        #     response.body = 'authentication failed'

    def __verify_user(self,user,password):
        #return user == 'cyt' and password == '123456'
        try:
            up = user_pass.UserPass()
            up.check_pass_word(user,password)
        except user_pass.NoUserException:
            return False



    @classmethod
    def factory(cls, global_conf, **kwargs):
        return cls()




import os
import sys
from oslo_config import cfg

PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '..'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
import select
import logging
import traceback
from logging.handlers import RotatingFileHandler
from common import epoll_recever
from common import acceptor
import conf
import worker_manager

logger = logging.getLogger('my_logger')

ID = 1
def run():
    recver = epoll_recever.Epoll_receiver()

    outer_ip = cfg.CONF.SERVER_IP
    outer_port = cfg.CONF.SERVER_PORT

    logger.info("Outer ip:%s:%d" % (outer_ip, outer_port))
    global ID
    logger.info(str(ID))
    ID = ID +1

    _worker_manager = worker_manager.WorkerManager(recver)
    _worker_manager.add_outer_worker(outer_ip,outer_port)

    while True:
        _worker_manager.all_do()
        recver.run()


def log_config(level):
    if not os.path.isdir('/var/log/tcp_forward'):
        os.makedirs('/var/log/tcp_forward')
    logger.setLevel(logging._levelNames[level])
    handler = RotatingFileHandler("/var/log/tcp_forward/forward_client.log", maxBytes=10000000, backupCount=10)
    console = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s %(process)d %(levelname)s %(filename)s:%(lineno)s %(funcName)s [-] %(message)s ')
    handler.setFormatter(formatter)
    console.setFormatter(formatter)
    logger.addHandler(handler)
    logger.addHandler(console)


def main():
    conf.register_core_common_config_opts(cfg.CONF)

    log_config(cfg.CONF.LOG_LEVEL)

    logger.info("Process start!")
    run()


if __name__ == '__main__':
    cfg.CONF(sys.argv[1:])
    sys.exit(main())


import forward_data, protocol_handler
import connector
import logging
logger = logging.getLogger('my_logger')


class DataHandler(object):
    def __init__(self):
        pass

    def parse_data(self,ring_buffer):
        """
        Use protocol handler to get one complete package
        When we get a timeout exception ,means there is still some data not received due to the the characteristics of tcp flow
        But when we lost data due to tcp stack or some other reasons,perhaps we will enter a forever loop
        The loop's creation like this:
            1.Receive a half package ,the package tell me there are 1M data in the package
            2.Receive data in the following loop until 1M is full
            3.It's virtually impossible to recevie all data because the following data have losted
            4.So i use timeout ring_buffer to record parse_timeout seconds,delete the package which receive time over 5 seconds
        """
        #get one complete package

        if ring_buffer.buf_len() <= 0:
            return None


        protocol_parse = protocol_handler.ProtocolHandler()
        package = None
        try:
            protocol_parse.del_wrong_header(ring_buffer)
            package = protocol_parse.get_one_complete_package(ring_buffer)
            ring_buffer.set_clear()
        except protocol_handler.ParseTimeout,e:
            #print 'parse timeout,current buffer:'
            #logger.debug(e.message)
            ring_buffer.set_time_out()
            if ring_buffer.over_max_time():
                protocol_parse.pop_header(ring_buffer)
                protocol_parse.del_wrong_header(ring_buffer)
                logger.error('ring buffer over max timeout time,del wrong header')
            return None
        except protocol_handler.ParseError,e:
            print 'parse error'
            protocol_parse.pop_header(ring_buffer)
            return None
        except Exception,e:
            return None


        if package == None:
            print 'package is None'
            return None

        parsed_data = protocol_parse.parse_data(package)

        if parsed_data == None:
            print 'parse package failed'
            return None

        return parsed_data

    def get_forward_datas(self, ring_buffer):
        forward_datas = []
        while True:
            parsed_data = self.parse_data(ring_buffer)
            if parsed_data == None:
                return tuple(forward_datas)
            forward_datas.append(parsed_data)


import socket,select
import errno

EVENT_TYPE = 'EVENT_TYPE'
EVENT_HANDLER = 'EVENT_HANDLER'
class Epoll_receiver(object):



    def __init__(self):
        self.__recv_fds = {}
        self.__epoll = select.epoll()

    def add_receiver(self,fd,events,event_handler):
        event_info = {
            EVENT_TYPE:events,
            EVENT_HANDLER:event_handler
        }
        if not self.__recv_fds.has_key(fd):
            self.__recv_fds[fd] = event_info
            self.__epoll.register(fd, events)
        else:
            self.change_event(fd,events,event_handler)

    def change_event(self,fd,events,event_handler):
        event_info = {
            EVENT_TYPE: events,
            EVENT_HANDLER: event_handler
        }
        if self.__recv_fds.has_key(fd):
            try:
                self.__epoll.unregister(fd)
            except Exception,e:
                pass
            self.__epoll.register(fd,events)
            self.__recv_fds[fd] = event_info

    def del_receiver(self,fd):
        if self.__recv_fds.has_key(fd):
            self.__recv_fds.pop(fd)
            self.__epoll.unregister(fd)

    def run(self):
        events = self.__epoll.poll(1)
        for fileno, event in events:
            if not self.__recv_fds.has_key(fileno):
                continue

            self.__recv_fds[fileno][EVENT_HANDLER](event)




#!/usr/bin/env python
#-*- coding:utf-8 -*-

from setuptools import setup, find_packages

setup(
    name = "tcp-forward",
    version = "1.0.4",
    keywords = ("tcp-forward"),
    description = "forward data from an established tcp connection",
    license = "MIT",

    url = "https://github.com/cao19881125/tcp_forward.git",
    author = "yuntao.cao",
    author_email = "yuntao.cao@nocsys.com.cn",

    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = ['enum','argparse','ConfigParser','pyinotify','oslo_config','oslo_service','webob','gevent','itsdangerous'],
    data_files = [('/etc/tcp-forward',['./etc/forward_server.cfg','./etc/forward_client.cfg','./etc/port_mapper.cfg','./etc/api-paste.ini','./etc/user_file'])],
    scripts=['tools/tcp-forward']
)



import socket
import errno
from enum import Enum

class CON_STATE(Enum):
    CON_NONE = 0
    CON_CONNECTING = 1
    CON_CONNECTED = 2
    CON_FAILED = 3
    CON_CLOSED = 4


class Connector(object):

    def __init__(self,remote_socket):
        self._socket = remote_socket
        self.refresh_con_state()
        self._fileno = self._socket.fileno()

    def close(self):
        if self.con_state != CON_STATE.CON_CLOSED:
            self._socket.close()
            self.con_state = CON_STATE.CON_CLOSED

    def recv(self):
        if self.con_state != CON_STATE.CON_CONNECTED:
            return ''

        recv_msg = ''
        while True:
            try:
                recv_msg_temp = self._socket.recv(1024 * 1024)
                if len(recv_msg_temp) == 0:
                    # connect closed
                    self.con_state = CON_STATE.CON_CLOSED
                    return ''
                else:
                    recv_msg += recv_msg_temp
            except socket.error,e:
                err = e.args[0]
                if err is errno.EAGAIN:
                    #recv finished
                    return recv_msg
                else:
                    # connect closed
                    self._socket.close()
                    self.con_state = CON_STATE.CON_CLOSED
                    return ''

    def send(self,msg):
        try:
            self._socket.setblocking(1)
            send_bytes = self._socket.send(msg)
            self._socket.setblocking(0)
            return send_bytes
        except Exception,e:
            self._socket.close()
            self.con_state = CON_STATE.CON_CLOSED
            return 0

    def get_fileno(self):
        return self._fileno

    def refresh_con_state(self):
        ret = self._socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if ret == 0:
            self.con_state = CON_STATE.CON_CONNECTED
        else:
            self.con_state = CON_STATE.CON_FAILED


class ConnectorClient(Connector):
    def __init__(self,remote_ip,remote_port):
        self._remote_ip = remote_ip
        self._remote_port = remote_port
        self.con_state = CON_STATE.CON_NONE

    def connect(self):
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._socket.setblocking(0)
        self._fileno = self._socket.fileno()
        try:
            self._socket.connect((self._remote_ip, self._remote_port))
        except socket.error,e:
            if e.errno == errno.EINPROGRESS:
                self.con_state = CON_STATE.CON_CONNECTING
            else:
                self.con_state = CON_STATE.CON_FAILED
        else:
            self.con_state = CON_STATE.CON_CONNECTED

if __name__ == '__main__':
    import select
    epoll = select.epoll()

    con1 = Connector('127.0.0.1',1234)
    con2 = Connector('127.0.0.1',1235)


    con1.connect()
    con2.connect()

    epoll.register(con1.get_fileno(), select.EPOLLIN)
    epoll.register(con2.get_fileno(), select.EPOLLIN)

    while True:
        events = epoll.poll(1)
        for fileno, event in events:
            if fileno == con1.get_fileno():
                if con1.con_state == CON_STATE.CON_CONNECTING:
                    con1.refresh_con_state()
                elif con1.con_state == CON_STATE.CON_CONNECTED:
                    recv_msg = con1.recv()
                    con2.send(recv_msg)
            elif fileno == con2.get_fileno():
                if con2.con_state == CON_STATE.CON_CONNECTING:
                    con2.refresh_con_state()
                elif con2.con_state == CON_STATE.CON_CONNECTED:
                    recv_msg = con2.recv()
                    con1.send(recv_msg)



import ConfigParser
from oslo_config import cfg


class NoUserException(Exception):
    pass

class PassErrorException(Exception):
    pass

class UserPass(object):
    def __init__(self):
        pass


    def check_pass_word(self,user,password):
        all_users = self.__parse_file()
        if all_users.has_key(user):
            if all_users[user] == password:
                return True
            else:
                raise PassErrorException()
        else:
            raise NoUserException()

    def __parse_file(self):
        cf = ConfigParser.ConfigParser()
        all_users = {}
        try:
            cf.readfp(open(cfg.CONF.user_file,'rb'))
            users = cf.items('USER')
            for item in users:
                user = item[0].lstrip().rstrip()
                passwd = item[1].lstrip().rstrip()
                all_users[user] = passwd

        except Exception,e:
            pass

        return all_users


from enum import Enum
from common import connector, epoll_recever, ring_buffer
import logging
import select
from logging.handlers import RotatingFileHandler
import traceback
import data_handler
from common import forward_event
from common import forward_data

logger = logging.getLogger('my_logger')

class OuterWorker(object):
    class State(Enum):
        NONE = 0
        CONNECTING = 1
        WORKING = 2
        DISCONNECTED = 3
    def __init__(self,outer_ip,outer_port,sourth_interface_channel,connector_change_callback):
        self.__state = self.State.NONE
        self.__outer_ip = outer_ip
        self.__outer_port = outer_port
        self.__connector = None
        self.__ring_buffer = ring_buffer.TimeoutRingbuffer(10240 * 10240, 5)
        self.__data_handler = data_handler.OuterDataHandler()
        self.__sourth_interface_channel = sourth_interface_channel
        self.__connector_change_callback = connector_change_callback

    def __north_interface_event(self, event):
        if self.__state == self.State.CONNECTING:
            self.__connector.refresh_con_state()

            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.WORKING
                self.__connector_change_callback(self.__connector,self.handler_event)
                logger.info("OuterWorker connect success change state to WORKING")

            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                self.__connector.close()
                self.__connector_change_callback(self.__connector,self.handler_event)
                self.__state = self.State.DISCONNECTED
                logger.error("OuterWorker connect failed,change state to DISCONNECTED")
        elif self.__state == self.State.WORKING:
            self.__handle_working_event(event)

    def __handle_working_event(self,event):
        #print '__handle_working_event'
        error_happen = False
        if event.fd_event & select.EPOLLIN:
            recv_msg = self.__connector.recv()
            if len(recv_msg) > 0:
                #pass data
                self.__ring_buffer.put(bytearray(recv_msg))
                #self.__ring_buffer.print_buf()
            else:
                if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                    logger.error("OuterWorker current state:WORKING recv data error")

        elif event.fd_event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self.__connector.close()
            self.__connector_change_callback(self.__connector,self.handler_event)
            self.__state = self.State.DISCONNECTED
            logger.debug("OuterWorkercurrent state:WORKING change state to DISCONNECTED")

    def __handle_data(self):
        datas = self.__data_handler.get_forward_datas(self.__ring_buffer)
        for data in datas:
            if data.data_type in (forward_data.DATA_TYPE.NEW_CONNECTION,
                                  forward_data.DATA_TYPE.TRANS_DATA):
                trans_event = forward_event.TransDataEvent(data.id,data)
                self.__sourth_interface_channel(trans_event)
            elif data.data_type == forward_data.DATA_TYPE.PRE_WORKING:
                self.__data_handler.send_preworking_reply(self.__connector)
            elif data.data_type == forward_data.DATA_TYPE.CLOSE_CONNECTION:
                close_event = forward_event.CloseConEvent(data.id)
                self.__sourth_interface_channel(close_event)

    def __sourth_interface_transdata_event(self, event):
        if not isinstance(event,forward_event.TransDataEvent):
            return
        f_data = event.forward_data

        if event.forward_data.data_type == forward_data.DATA_TYPE.CONNECTION_SUCCESS:
            self.__data_handler.inner_connect_succed(f_data.id,f_data.inner_ip,f_data.inner_port,self.__connector)
        elif event.forward_data.data_type == forward_data.DATA_TYPE.TRANS_DATA:
            self.__data_handler.trans_data(f_data.id,f_data.inner_ip,f_data.inner_port,f_data.data,self.__connector)

    def __sourth_interface_closecon_event(self, event):
        if not isinstance(event,forward_event.CloseConEvent):
            return
        self.__data_handler.close_connection(event.forward_id,'0.0.0.0',0,self.__connector)

    def __scheduler_event(self, event):

        if not isinstance(event,forward_event.SchedulerEvent):
            return


        if self.__state == self.State.NONE:
            #print 'outer_worker:connect to %s %d'%(self.__outer_ip,self.__outer_port)
            self.__connector = connector.ConnectorClient(self.__outer_ip, self.__outer_port)
            self.__connector.connect()

            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                self.__connector_change_callback(self.__connector,self.handler_event)
                self.__state = self.State.WORKING
            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                #do nothing
                pass
            elif self.__connector.con_state == connector.CON_STATE.CON_CONNECTING:
                self.__connector_change_callback(self.__connector,self.handler_event)
                self.__state = self.State.CONNECTING
        elif self.__state == self.State.WORKING:
            if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.DISCONNECTED
                logger.debug("OuterWorker current state:WORKING change state to DONE due connector state error:%s"%(str(self.__connector.con_state)) )
                return
        elif self.__state == self.State.DISCONNECTED:
            close_event = forward_event.CloseConEvent(0)
            self.__sourth_interface_channel(close_event)
            self.__state = self.State.NONE
            logger.debug("OuterWorker current state:DISCONNECTED close all inner_worker,change state to NONE")
        self.__handle_data()

    @forward_event.event_filter
    def handler_event(self, event):
        if event.event_type == forward_event.FDEVENT:
            # socket receive msg
            self.__north_interface_event(event)
        elif event.event_type == forward_event.TRANSDATAEVENT:
            self.__sourth_interface_transdata_event(event)
        elif event.event_type == forward_event.CLOSECONEVENT:
            self.__sourth_interface_closecon_event(event)
        elif event.event_type == forward_event.SCHEDULEREVENT:
            self.__scheduler_event(event)


import sys
import os
PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '.'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
from webob import Request
from oslo_config import cfg
from oslo_service import wsgi
from gevent import pywsgi
import threading
import forward_server


class ShowVersion():
    def __init__(self):
        pass
    def __call__(self,environ,start_response):
        start_response("200 OK",[("Content-type", "text/plain")])
        #print environ
        req = Request(environ)
        print req.method
        print req.POST
        print req.body
        print req.authorization
        
        for key,value in req.params.items():
            print "key=" + str(key) + "  value=" + str(value)
        return ["Paste Deploy LAB: Version = 1.0.0",]
    @classmethod
    def factory(cls,global_conf,**kwargs):
        return ShowVersion()



def main():
    t = threading.Thread(target=forward_server.main)
    t.setDaemon(True)
    t.start()

    loader = wsgi.Loader(cfg.CONF)
    wsgi_app = loader.load_app('forward-server')

    server = pywsgi.WSGIServer(('0.0.0.0',8080),wsgi_app)
    server.serve_forever()


if __name__ == '__main__':
    cfg.CONF(sys.argv[1:])
    sys.exit(main())
